@@ -1894,7 +1894,6 @@ EOT
       is used for retrieval, so anything that is a valid file source can
       be used here.",
     },
-
     :pluginsync => {
       :default    => true,
       :type       => :boolean,
@@ -1908,6 +1907,13 @@ EOT
     :pluginsignore => {
         :default  => ".svn CVS .git .hg",
         :desc     => "What files to ignore when pulling down plugins.",
+    },
+    :ignore_plugin_errors => {
+      :default    => true,
+      :type       => :boolean,
+      :desc       => "Whether the puppet run should ignore errors during pluginsync. If the setting
+        is false and there are errors during pluginsync, then the agent will abort the run and
+        submit a report containing information about the failed run."
     }
   )
 
