@@ -23,7 +23,7 @@ class Puppet::FileServing::HttpMetadata < Puppet::FileServing::Metadata
 
     if last_modified = http_response['last-modified']
       mtime = DateTime.httpdate(last_modified).to_time
-      @checksums[:mtime] = "{mtime}#{mtime}"
+      @checksums[:mtime] = "{mtime}#{mtime.utc}"
     end
 
     @ftype = 'file'
@@ -37,10 +37,10 @@ class Puppet::FileServing::HttpMetadata < Puppet::FileServing::Metadata
   def collect
     # Prefer the checksum_type from the indirector request options
     # but fall back to the alternative otherwise
-    [ @checksum_type, :md5, :mtime ].each do |type|
+    [ @checksum_type, :md5, :sha256, :sha384, :sha512, :sha224, :mtime ].each do |type|
       @checksum_type = type
       @checksum = @checksums[type]
-      return if @checksum
+      break if @checksum
     end
   end
 end
