@@ -72,7 +72,9 @@ module Manager
   def newtype(name, options = {}, &block)
     # Handle backward compatibility
     unless options.is_a?(Hash)
-      Puppet.warning "Puppet::Type.newtype(#{name}) now expects a hash as the second argument, not #{options.inspect}"
+      #TRANSLATORS 'Puppet::Type.newtype' should not be translated
+      Puppet.warning(_("Puppet::Type.newtype(%{name}) now expects a hash as the second argument, not %{argument}") %
+                         { name: name, argument: options.inspect})
     end
 
     # First make sure we don't have a method sitting around
@@ -107,7 +109,8 @@ module Manager
     # Now define a "new<type>" method for convenience.
     if self.respond_to? newmethod
       # Refuse to overwrite existing methods like 'newparam' or 'newtype'.
-      Puppet.warning "'new#{name.to_s}' method already exists; skipping"
+      #TRANSLATORS 'new%{method}' will become a method name, do not translate this string
+      Puppet.warning(_("'new%{method}' method already exists; skipping") % { method: name.to_s })
     else
       selfobj.send(:define_method, newmethod) do |*args|
         klass.new(*args)
@@ -120,7 +123,7 @@ module Manager
 
     # Now set up autoload any providers that might exist for this type.
 
-    klass.providerloader = Puppet::Util::Autoload.new(klass, "puppet/provider/#{klass.name.to_s}")
+    klass.providerloader = Puppet::Util::Autoload.new(klass, "puppet/provider/#{klass.name}")
 
     # We have to load everything so that we can figure out the default provider.
     klass.providerloader.loadall Puppet.lookup(:current_environment)
@@ -169,7 +172,8 @@ module Manager
     end
     # Try loading the type.
     if typeloader.load(name, Puppet.lookup(:current_environment))
-      Puppet.warning "Loaded puppet/type/#{name} but no class was created" unless @types.include? name
+      #TRANSLATORS 'puppet/type/%{name}' should not be translated
+      Puppet.warning(_("Loaded puppet/type/%{name} but no class was created") % { name: name }) unless @types.include? name
     elsif !Puppet[:always_retry_plugins]
       # PUP-5482 - Only look for a type once if plugin retry is disabled
       @types[name] = nil
