@@ -48,7 +48,7 @@ class Puppet::Node::Environment
     obj.send(:initialize,
              name.intern,
              expand_dirs(extralibs() + modulepath),
-             manifest == NO_MANIFEST ? manifest : File.expand_path(manifest),
+             manifest == NO_MANIFEST ? manifest : Puppet::FileSystem.expand_path(manifest),
              config_version)
     obj
   end
@@ -85,6 +85,7 @@ class Puppet::Node::Environment
     @modulepath = modulepath
     @manifest = manifest
     @config_version = config_version
+    @modules_strict_semver = false
   end
 
   # Creates a new Puppet::Node::Environment instance, overriding any of the passed
@@ -166,6 +167,10 @@ class Puppet::Node::Environment
   #     (optional)
   attr_reader :config_version
 
+  # Cached loaders - management of value handled by Puppet::Pops::Loaders
+  # @api private
+  attr_accessor :loaders
+
   # Checks to make sure that this environment did not have a manifest set in
   # its original environment.conf if Puppet is configured with
   # +disable_per_environment_manifest+ set true.  If it did, the environment's
@@ -209,11 +214,22 @@ class Puppet::Node::Environment
   def validation_errors
     errors = []
     if conflicting_manifest_settings?
-      errors << "The 'disable_per_environment_manifest' setting is true, and the '#{name}' environment has an environment.conf manifest that conflicts with the 'default_manifest' setting."
+      errors << _("The 'disable_per_environment_manifest' setting is true, and the '%{env_name}' environment has an environment.conf manifest that conflicts with the 'default_manifest' setting.") % { env_name: name }
     end
     errors
   end
 
+  # Checks if this environment permits use of rich data types in the catalog
+  # @return [Boolean] `true` if rich data is permitted.
+  # @api private
+  def rich_data?
+    if @rich_data.nil?
+      environment_conf = Puppet.lookup(:environments).get_conf(name)
+      @rich_data = (environment_conf.nil? ? Puppet[:rich_data] : environment_conf.rich_data)
+    end
+    @rich_data
+  end
+
   # Return an environment-specific Puppet setting.
   #
   # @api public
@@ -224,6 +240,22 @@ class Puppet::Node::Environment
     Puppet.settings.value(param, self.name)
   end
 
+  # A SemanticPuppet::VersionRange version >= 1.0.0 will not include versions with pre-release
+  # identifiers unless that is explicitly declared. This may cause backward compatibility
+  # issues when resolving module dependencies and the flag is therefore set to `false` by default.
+  #
+  # @param flag [Boolean] set to true to resolve module dependencies using strict SemVer semantics
+  #
+  def modules_strict_semver=(flag)
+    @modules_strict_semver = flag
+  end
+
+  # @return [Boolean] the current value of the modules_strict_semver flag.
+  # @api public
+  def modules_strict_semver?
+    @modules_strict_semver
+  end
+
   # @api public
   # @return [Puppet::Resource::TypeCollection] The current global TypeCollection
   def known_resource_types
@@ -253,7 +285,7 @@ class Puppet::Node::Environment
   # @param name [String] The module name
   # @return [Puppet::Module, nil] The module if found, else nil
   def module(name)
-    modules.find {|mod| mod.name == name}
+    modules_by_name[name]
   end
 
   # Locate a module instance by the full forge name (EG authorname/module)
@@ -263,7 +295,7 @@ class Puppet::Node::Environment
   # @param forge_name [String] The module name
   # @return [Puppet::Module, nil] The module if found, else nil
   def module_by_forge_name(forge_name)
-    author, modname = forge_name.split('/')
+    _, modname = forge_name.split('/')
     found_mod = self.module(modname)
     found_mod and found_mod.forge_name == forge_name ?
       found_mod :
@@ -298,7 +330,7 @@ class Puppet::Node::Environment
 
       @modules = module_references.collect do |reference|
         begin
-          Puppet::Module.new(reference[:name], reference[:path], self)
+          Puppet::Module.new(reference[:name], reference[:path], self, modules_strict_semver?)
         rescue Puppet::Module::Error => e
           Puppet.log_exception(e)
           nil
@@ -308,6 +340,12 @@ class Puppet::Node::Environment
     @modules
   end
 
+  # @api private
+  def modules_by_name
+    @modules_by_name ||= Hash[modules.map { |mod| [mod.name, mod] }]
+  end
+  private :modules_by_name
+
   # Generate a warning if the given directory in a module path entry is named `lib`.
   #
   # @api private
@@ -340,7 +378,7 @@ class Puppet::Node::Environment
             Puppet::Module.is_module_directory?(name, path)
           end
           modules_by_path[path] = module_names.sort.map do |name|
-            Puppet::Module.new(name, File.join(path, name), self)
+            Puppet::Module.new(name, File.join(path, name), self, modules_strict_semver?)
           end
         end
       else
@@ -401,6 +439,26 @@ class Puppet::Node::Environment
     deps
   end
 
+  # Loads module translations for the current environment once for
+  # the lifetime of the environment. Execute a block in the context
+  # of that translation domain.
+  def with_text_domain
+    return yield if Puppet[:disable_i18n]
+
+    if @text_domain.nil?
+      @text_domain = @name
+      Puppet::GettextConfig.reset_text_domain(@text_domain)
+      Puppet::ModuleTranslations.load_from_modulepath(modules)
+    else
+      Puppet::GettextConfig.use_text_domain(@text_domain)
+    end
+
+    yield
+  ensure
+    # Is a noop if disable_i18n is true
+    Puppet::GettextConfig.clear_text_domain
+  end
+
   # Checks if a reparse is required (cache of files is stale).
   #
   def check_for_reparse
@@ -455,8 +513,7 @@ class Puppet::Node::Environment
     [self.class, name, full_modulepath, manifest].hash
   end
 
-  private
-
+  # not private so it can be called in tests
   def self.extralibs()
     if Puppet::Util.get_env('PUPPETLIB')
       split_path(Puppet::Util.get_env('PUPPETLIB'))
@@ -465,11 +522,14 @@ class Puppet::Node::Environment
     end
   end
 
+  private
+
   def self.expand_dirs(dirs)
     dirs.collect do |dir|
-      File.expand_path(dir)
+      Puppet::FileSystem.expand_path(dir)
     end
   end
+  private_class_method :expand_dirs
 
   # Reparse the manifests for the given environment
   #
@@ -515,13 +575,13 @@ class Puppet::Node::Environment
   rescue => detail
     @known_resource_types.parse_failed = true
 
-    msg = "Could not parse for environment #{self}: #{detail}"
+    msg = _("Could not parse for environment %{env}: %{detail}") % { env: self, detail: detail }
     error = Puppet::Error.new(msg)
     error.set_backtrace(detail.backtrace)
     raise error
   end
 
-  # Return an empty toplevel hostclass to indicate that no file was loaded
+  # Return an empty top-level hostclass to indicate that no file was loaded
   #
   # @return [Puppet::Parser::AST::Hostclass]
   def empty_parse_result
