@@ -31,12 +31,14 @@ class ValidatorFactory_4_0 < Factory
     p[Issues::RT_NO_STORECONFIGS]            = Puppet[:storeconfigs] ? :ignore : :warning
 
     p[Issues::FUTURE_RESERVED_WORD]          = :deprecation
+    p[Issues::DEPRECATED_APP_ORCHESTRATION]  = :deprecation
 
     p[Issues::DUPLICATE_KEY]                 = Puppet[:strict] == :off ? :ignore : Puppet[:strict]
     p[Issues::NAME_WITH_HYPHEN]              = :error
     p[Issues::EMPTY_RESOURCE_SPECIALIZATION] = :ignore
     p[Issues::CLASS_NOT_VIRTUALIZABLE]       = Puppet[:strict] == :off ? :warning : Puppet[:strict]
     p[Issues::ILLEGAL_DEFINITION_LOCATION]   = :deprecation
+    p[Issues::ILLEGAL_TOP_CONSTRUCT_LOCATION]   = :deprecation
     p
   end
 end
