@@ -1,6 +1,7 @@
 require 'puppet/provider/exec'
 
 Puppet::Type.type(:exec).provide :posix, :parent => Puppet::Provider::Exec do
+  has_feature :umask
   confine :feature => :posix
   defaultfor :feature => :posix
 
@@ -16,12 +17,12 @@ Puppet::Type.type(:exec).provide :posix, :parent => Puppet::Provider::Exec do
     exe = extractexe(command)
 
     if File.expand_path(exe) == exe
-      if !File.exists?(exe)
-        raise ArgumentError, "Could not find command '#{exe}'"
+      if !Puppet::FileSystem.exist?(exe)
+        raise ArgumentError, _("Could not find command '%{exe}'") % { exe: exe }
       elsif !File.file?(exe)
-        raise ArgumentError, "'#{exe}' is a #{File.ftype(exe)}, not a file"
+        raise ArgumentError, _("'%{exe}' is a %{klass}, not a file") % { exe: exe, klass: File.ftype(exe) }
       elsif !File.executable?(exe)
-        raise ArgumentError, "'#{exe}' is not executable"
+        raise ArgumentError, _("'%{exe}' is not executable") % { exe: exe }
       end
       return
     end
@@ -34,6 +35,14 @@ Puppet::Type.type(:exec).provide :posix, :parent => Puppet::Provider::Exec do
 
     # 'which' will only return the command if it's executable, so we can't
     # distinguish not found from not executable
-    raise ArgumentError, "Could not find command '#{exe}'"
+    raise ArgumentError, _("Could not find command '%{exe}'") % { exe: exe }
+  end
+
+  def run(command, check = false)
+    if resource[:umask]
+      Puppet::Util::withumask(resource[:umask]) { super(command, check) }
+    else
+      super(command, check)
+    end
   end
 end
