@@ -20,7 +20,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
   has_feature :install_options
   has_feature :uninstall_options
   has_feature :upgradeable
-  has_feature :flavorable
+  has_feature :supports_flavors
 
   mk_resource_methods
 
@@ -33,7 +33,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
       packages = pkginfo('-a')
       packages.split("\n").collect do |package|
         if match = regex.match(package.split[0])
-         new( :name => match.captures[0],
+          new( :name => match.captures[0],
                :ensure => match.captures[1],
                :flavor => match.captures[2],
           )
@@ -145,7 +145,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
       # If :ensure contains a version, use that instead of looking it up.
       # This allows for installing packages with the same stem, but multiple
       # version such as openldap-server.
-      if /(\d[^-]*)$/.match(@resource[:ensure].to_s)
+      if @resource[:ensure].to_s =~ /(\d[^-]*)$/
         use_version = @resource[:ensure]
       else
  
@@ -205,4 +205,15 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
   def purge
     pkgdelete "-c", "-q", @resource[:name]
   end
+
+  def flavor
+    @property_hash[:flavor]
+  end
+
+  def flavor=(value)
+    if flavor != @resource.should(:flavor)
+      uninstall
+      install
+    end
+  end
 end
