@@ -77,7 +77,7 @@ Puppet::Type.type(:package).provide :pip, :parent => ::Puppet::Provider::Package
       command_options << '--all'
     end
 
-    execpipe [command, command_options] do |process|
+    execpipe [quote(command), command_options] do |process|
       process.collect do |line|
         next unless pkg = parse(line)
         pkg[:command] = command
@@ -138,7 +138,7 @@ Puppet::Type.type(:package).provide :pip, :parent => ::Puppet::Provider::Package
     command = resource_or_provider_command
     self.class.validate_command(command)
 
-    command_and_options = [command, 'install', "#{@resource[:name]}==versionplease"]
+    command_and_options = [self.class.quote(command), 'install', "#{@resource[:name]}==versionplease"]
     command_and_options << install_options if @resource[:install_options]
     execpipe command_and_options do |process|
       process.collect do |line|
@@ -162,7 +162,7 @@ Puppet::Type.type(:package).provide :pip, :parent => ::Puppet::Provider::Package
     self.class.validate_command(command)
 
     Dir.mktmpdir("puppet_pip") do |dir|
-      command_and_options = [command, 'install', "#{@resource[:name]}", '-d', "#{dir}", '-v']
+      command_and_options = [self.class.quote(command), 'install', "#{@resource[:name]}", '-d', "#{dir}", '-v']
       command_and_options << install_options if @resource[:install_options]
       execpipe command_and_options do |process|
         process.collect do |line|
@@ -227,6 +227,9 @@ Puppet::Type.type(:package).provide :pip, :parent => ::Puppet::Provider::Package
     join_options(@resource[:install_options])
   end
 
+  # Quoting is required if the path to the pip command contains spaces.
+  # Required for execpipe() but not execute(), as execute() already does this.
+
   def self.quote(path)
     if path.include?(" ")
       "\"#{path}\""
@@ -234,5 +237,4 @@ Puppet::Type.type(:package).provide :pip, :parent => ::Puppet::Provider::Package
       path
     end
   end
-  private_class_method :quote
 end
