@@ -97,6 +97,7 @@ module Puppet::Util::Windows::Security
   FILE = Puppet::Util::Windows::File
 
   SE_BACKUP_NAME              = 'SeBackupPrivilege'
+  SE_DEBUG_NAME               = 'SeDebugPrivilege'
   SE_RESTORE_NAME             = 'SeRestorePrivilege'
 
   DELETE                      = 0x00010000
@@ -170,7 +171,7 @@ module Puppet::Util::Windows::Security
       if GetVolumeInformationW(wide_string(root), FFI::Pointer::NULL, 0,
           FFI::Pointer::NULL, FFI::Pointer::NULL,
           flags_ptr, FFI::Pointer::NULL, 0) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Failed to get volume information")
+        raise Puppet::Util::Windows::Error.new(_("Failed to get volume information"))
       end
       supported = flags_ptr.read_dword & FILE_PERSISTENT_ACLS == FILE_PERSISTENT_ACLS
     end
@@ -200,6 +201,7 @@ module Puppet::Util::Windows::Security
     well_known_world_sid = Puppet::Util::Windows::SID::Everyone
     well_known_nobody_sid = Puppet::Util::Windows::SID::Nobody
     well_known_system_sid = Puppet::Util::Windows::SID::LocalSystem
+    well_known_app_packages_sid = Puppet::Util::Windows::SID::AllAppPackages
 
     mode = S_ISYSTEM_MISSING
 
@@ -234,6 +236,7 @@ module Puppet::Util::Windows::Security
         if (ace.mask & FILE::FILE_APPEND_DATA).nonzero?
           mode |= S_ISVTX
         end
+      when well_known_app_packages_sid
       when well_known_system_sid
       else
         #puts "Warning, unable to map SID into POSIX mode: #{ace.sid}"
@@ -262,7 +265,7 @@ module Puppet::Util::Windows::Security
   }
 
   # Set the mode of the object referenced by +path+ to the specified
-  # +mode+.  The mode should be specified as POSIX-stye read, write,
+  # +mode+. The mode should be specified as POSIX-style read, write,
   # and execute modes for the user, group, and other classes,
   # e.g. 0640. The sticky bit, S_ISVTX, is supported, but is only
   # meaningful for directories. If set, group and others are not
@@ -274,7 +277,7 @@ module Puppet::Util::Windows::Security
   # mode. Only a user with the SE_BACKUP_NAME and SE_RESTORE_NAME
   # privileges in their process token can change the mode for objects
   # that they do not have read and write access to.
-  def set_mode(mode, path, protected = true)
+  def set_mode(mode, path, protected = true, managing_owner = false, managing_group = false)
     sd = get_security_descriptor(path)
     well_known_world_sid = Puppet::Util::Windows::SID::Everyone
     well_known_nobody_sid = Puppet::Util::Windows::SID::Nobody
@@ -283,6 +286,8 @@ module Puppet::Util::Windows::Security
     owner_allow = FILE::STANDARD_RIGHTS_ALL  |
       FILE::FILE_READ_ATTRIBUTES |
       FILE::FILE_WRITE_ATTRIBUTES
+    # this prevents a mode that is not 7 from taking ownership of a file based
+    # on group membership and rewriting it / making it executable
     group_allow = FILE::STANDARD_RIGHTS_READ |
       FILE::FILE_READ_ATTRIBUTES |
       FILE::SYNCHRONIZE
@@ -304,33 +309,75 @@ module Puppet::Util::Windows::Security
       end
     end
 
+    # With a mode value of '7' for group / other, the value must then include
+    # additional perms beyond STANDARD_RIGHTS_READ to allow DACL modification
+    if ((mode & S_IRWXG) == S_IRWXG)
+      group_allow |= FILE::DELETE | FILE::WRITE_DAC | FILE::WRITE_OWNER
+    end
+    if ((mode & S_IRWXO) == S_IRWXO)
+      other_allow |= FILE::DELETE | FILE::WRITE_DAC | FILE::WRITE_OWNER
+    end
+
     if (mode & S_ISVTX).nonzero?
       nobody_allow |= FILE::FILE_APPEND_DATA;
     end
 
+    isownergroup = sd.owner == sd.group
+
     # caller is NOT managing SYSTEM by using group or owner, so set to FULL
     if ! [sd.owner, sd.group].include? well_known_system_sid
       # we don't check S_ISYSTEM_MISSING bit, but automatically carry over existing SYSTEM perms
       # by default set SYSTEM perms to full
       system_allow = FILE::FILE_ALL_ACCESS
+    else
+      # It is possible to set SYSTEM with a mode other than Full Control (7) however this makes no sense and in practical terms
+      # should not be done.  We can trap these instances and correct them before being applied.
+      if (sd.owner == well_known_system_sid) && (owner_allow != FILE::FILE_ALL_ACCESS)
+        # If owner and group are both SYSTEM but group is unmanaged the control rights of system will be set to FullControl by
+        # the unmanaged group, so there is no need for the warning
+        if managing_owner && (!isownergroup || managing_group)
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.warning _("Setting control rights for %{path} owner SYSTEM to less than Full Control rights. Setting SYSTEM rights to less than Full Control may have unintented consequences for operations on this file") % { path: path }
+        elsif managing_owner && isownergroup
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.debug _("%{path} owner and group both set to user SYSTEM, but group is not managed directly: SYSTEM user rights will be set to FullControl by group") % { path: path }
+        else
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.debug _("An attempt to set mode %{mode} on item %{path} would result in the owner, SYSTEM, to have less than Full Control rights. This attempt has been corrected to Full Control") % { mode: mode.to_s(8), path: path }
+          owner_allow = FILE::FILE_ALL_ACCESS
+        end
+      end
+
+      if (sd.group == well_known_system_sid) && (group_allow != FILE::FILE_ALL_ACCESS)
+        # If owner and group are both SYSTEM but owner is unmanaged the control rights of system will be set to FullControl by
+        # the unmanaged owner, so there is no need for the warning.
+        if managing_group && (!isownergroup || managing_owner)
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.warning _("Setting control rights for %{path} group SYSTEM to less than Full Control rights. Setting SYSTEM rights to less than Full Control may have unintented consequences for operations on this file") % { path: path }
+        elsif managing_group && isownergroup
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.debug _("%{path} owner and group both set to user SYSTEM, but owner is not managed directly: SYSTEM user rights will be set to FullControl by owner") % { path: path }
+        else
+          #TRANSLATORS 'SYSTEM' is a Windows name and should not be translated
+          Puppet.debug _("An attempt to set mode %{mode} on item %{path} would result in the group, SYSTEM, to have less than Full Control rights. This attempt has been corrected to Full Control") % { mode: mode.to_s(8), path: path }
+          group_allow = FILE::FILE_ALL_ACCESS
+        end
+      end
     end
 
-    isdir = File.directory?(path)
-
-    if isdir
-      if (mode & (S_IWUSR | S_IXUSR)) == (S_IWUSR | S_IXUSR)
-        owner_allow |= FILE::FILE_DELETE_CHILD
-      end
-      if (mode & (S_IWGRP | S_IXGRP)) == (S_IWGRP | S_IXGRP) && (mode & S_ISVTX) == 0
-        group_allow |= FILE::FILE_DELETE_CHILD
-      end
-      if (mode & (S_IWOTH | S_IXOTH)) == (S_IWOTH | S_IXOTH) && (mode & S_ISVTX) == 0
-        other_allow |= FILE::FILE_DELETE_CHILD
-      end
+    # even though FILE_DELETE_CHILD only applies to directories, it can be set on files
+    # this is necessary to do to ensure a file ends up with (F) FullControl
+    if (mode & (S_IWUSR | S_IXUSR)) == (S_IWUSR | S_IXUSR)
+      owner_allow |= FILE::FILE_DELETE_CHILD
+    end
+    if (mode & (S_IWGRP | S_IXGRP)) == (S_IWGRP | S_IXGRP) && (mode & S_ISVTX) == 0
+      group_allow |= FILE::FILE_DELETE_CHILD
+    end
+    if (mode & (S_IWOTH | S_IXOTH)) == (S_IWOTH | S_IXOTH) && (mode & S_ISVTX) == 0
+      other_allow |= FILE::FILE_DELETE_CHILD
     end
 
     # if owner and group the same, then map group permissions to the one owner ACE
-    isownergroup = sd.owner == sd.group
     if isownergroup
       owner_allow |= group_allow
     end
@@ -341,6 +388,7 @@ module Puppet::Util::Windows::Security
       FILE.remove_attributes(path, FILE::FILE_ATTRIBUTE_READONLY)
     end
 
+    isdir = File.directory?(path)
     dacl = Puppet::Util::Windows::AccessControlList.new
     dacl.allow(sd.owner, owner_allow)
     unless isownergroup
@@ -363,8 +411,10 @@ module Puppet::Util::Windows::Security
       dacl.allow(Puppet::Util::Windows::SID::CreatorGroup, group_allow, inherit)
 
       inherit = inherit_only | Puppet::Util::Windows::AccessControlEntry::OBJECT_INHERIT_ACE
-      dacl.allow(Puppet::Util::Windows::SID::CreatorOwner, owner_allow & ~FILE::FILE_EXECUTE, inherit)
-      dacl.allow(Puppet::Util::Windows::SID::CreatorGroup, group_allow & ~FILE::FILE_EXECUTE, inherit)
+      # allow any previously set bits *except* for these
+      perms_to_strip = ~(FILE::FILE_EXECUTE + FILE::WRITE_OWNER + FILE::WRITE_DAC)
+      dacl.allow(Puppet::Util::Windows::SID::CreatorOwner, owner_allow & perms_to_strip, inherit)
+      dacl.allow(Puppet::Util::Windows::SID::CreatorGroup, group_allow & perms_to_strip, inherit)
     end
 
     new_sd = Puppet::Util::Windows::SecurityDescriptor.new(sd.owner, sd.group, dacl, protected)
@@ -380,11 +430,11 @@ module Puppet::Util::Windows::Security
 
     Puppet::Util::Windows::SID.string_to_sid_ptr(sid) do |sid_ptr|
       if Puppet::Util::Windows::SID.IsValidSid(sid_ptr) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Invalid SID")
+        raise Puppet::Util::Windows::Error.new(_("Invalid SID"))
       end
 
       if AddAccessAllowedAceEx(acl, ACL_REVISION, inherit, mask, sid_ptr) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Failed to add access control entry")
+        raise Puppet::Util::Windows::Error.new(_("Failed to add access control entry"))
       end
     end
 
@@ -397,11 +447,11 @@ module Puppet::Util::Windows::Security
 
     Puppet::Util::Windows::SID.string_to_sid_ptr(sid) do |sid_ptr|
       if Puppet::Util::Windows::SID.IsValidSid(sid_ptr) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Invalid SID")
+        raise Puppet::Util::Windows::Error.new(_("Invalid SID"))
       end
 
       if AddAccessDeniedAceEx(acl, ACL_REVISION, inherit, mask, sid_ptr) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Failed to add access control entry")
+        raise Puppet::Util::Windows::Error.new(_("Failed to add access control entry"))
       end
     end
 
@@ -412,7 +462,7 @@ module Puppet::Util::Windows::Security
   def parse_dacl(dacl_ptr)
     # REMIND: need to handle NULL DACL
     if IsValidAcl(dacl_ptr) == FFI::WIN32_FALSE
-      raise Puppet::Util::Windows::Error.new("Invalid DACL")
+      raise Puppet::Util::Windows::Error.new(_("Invalid DACL"))
     end
 
     dacl_struct = ACL.new(dacl_ptr)
@@ -435,7 +485,7 @@ module Puppet::Util::Windows::Security
         ace_type = ace[:Header][:AceType]
         if ace_type != Puppet::Util::Windows::AccessControlEntry::ACCESS_ALLOWED_ACE_TYPE &&
           ace_type != Puppet::Util::Windows::AccessControlEntry::ACCESS_DENIED_ACE_TYPE
-          Puppet.warning "Unsupported access control entry type: 0x#{ace_type.to_s(16)}"
+          Puppet.warning _("Unsupported access control entry type: 0x%{type}") % { type: ace_type.to_s(16) }
           next
         end
 
@@ -469,7 +519,7 @@ module Puppet::Util::Windows::Security
              FFI::Pointer::NULL_HANDLE) # template
 
     if handle == Puppet::Util::Windows::File::INVALID_HANDLE_VALUE
-      raise Puppet::Util::Windows::Error.new("Failed to open '#{path}'")
+      raise Puppet::Util::Windows::Error.new(_("Failed to open '%{path}'") % { path: path })
     end
 
     begin
@@ -516,7 +566,7 @@ module Puppet::Util::Windows::Security
             if AdjustTokenPrivileges(token, FFI::WIN32_FALSE,
                 token_privileges, token_privileges.size,
                 FFI::MemoryPointer::NULL, FFI::MemoryPointer::NULL) == FFI::WIN32_FALSE
-              raise Puppet::Util::Windows::Error.new("Failed to adjust process privileges")
+              raise Puppet::Util::Windows::Error.new(_("Failed to adjust process privileges"))
             end
           end
         end
@@ -546,7 +596,7 @@ module Puppet::Util::Windows::Security
                   dacl_ptr_ptr,
                   FFI::Pointer::NULL, #sacl
                   sd_ptr_ptr) #sec desc
-                raise Puppet::Util::Windows::Error.new("Failed to get security information") if rv != FFI::ERROR_SUCCESS
+                raise Puppet::Util::Windows::Error.new(_("Failed to get security information")) if rv != FFI::ERROR_SUCCESS
 
                 # these 2 convenience params are not freed since they point inside sd_ptr
                 owner = Puppet::Util::Windows::SID.sid_ptr_to_string(owner_sid_ptr_ptr.get_pointer(0))
@@ -557,7 +607,7 @@ module Puppet::Util::Windows::Security
                     sd_ptr_ptr.read_win32_local_pointer do |sd_ptr|
 
                       if GetSecurityDescriptorControl(sd_ptr, control, revision) == FFI::WIN32_FALSE
-                        raise Puppet::Util::Windows::Error.new("Failed to get security descriptor control")
+                        raise Puppet::Util::Windows::Error.new(_("Failed to get security descriptor control"))
                       end
 
                       protect = (control.read_word & SE_DACL_PROTECTED) == SE_DACL_PROTECTED
@@ -590,11 +640,11 @@ module Puppet::Util::Windows::Security
   def set_security_descriptor(path, sd)
     FFI::MemoryPointer.new(:byte, get_max_generic_acl_size(sd.dacl.count)) do |acl_ptr|
       if InitializeAcl(acl_ptr, acl_ptr.size, ACL_REVISION) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Failed to initialize ACL")
+        raise Puppet::Util::Windows::Error.new(_("Failed to initialize ACL"))
       end
 
       if IsValidAcl(acl_ptr) == FFI::WIN32_FALSE
-        raise Puppet::Util::Windows::Error.new("Invalid DACL")
+        raise Puppet::Util::Windows::Error.new(_("Invalid DACL"))
       end
 
       with_privilege(SE_BACKUP_NAME) do
@@ -629,7 +679,7 @@ module Puppet::Util::Windows::Security
                                      FFI::MemoryPointer::NULL)
 
                 if rv != FFI::ERROR_SUCCESS
-                  raise Puppet::Util::Windows::Error.new("Failed to set security information")
+                  raise Puppet::Util::Windows::Error.new(_("Failed to set security information"))
                 end
               end
             end
